﻿using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.Xml.Linq;
using Microsoft.IdentityModel.Protocols.WSFederation.Metadata;
using Microsoft.IdentityModel.Protocols.WSIdentity;
using Microsoft.IdentityModel.SecurityTokenService;

namespace FirstIdentityProvider
{
    /// <summary>
    /// Summary description for metadata
    /// </summary>
    public class Metadata : IHttpHandler {

        public void ProcessRequest (HttpContext context) {
            context.Response.ContentType = "text/xml";

            // Entity
            var ent = new EntityDescriptor();
            ent.Contacts.Add(new ContactPerson(ContactType.Technical) { GivenName = "Pedro", Surname = "Felix" });
            ent.EntityId = new EntityId(SimpleSecurityTokenServiceConfiguration.IssuerName);
            ent.SigningCredentials = new X509SigningCredentials(SimpleSecurityTokenServiceConfiguration.SigningCertificate);

            // Role: ApplicationService
            var role = new SecurityTokenServiceDescriptor();
            var endpoints = new EndpointAddress[] {
                new EndpointAddress("https://sts.idp.ciws/issue.aspx"),                
            };
            foreach (var ep in endpoints)
            {
                role.SecurityTokenServiceEndpoints.Add(ep);
                role.PassiveRequestorEndpoints.Add(ep);
            }

            role.ServiceDescription = "sts.idp.ciws description";
            role.ServiceDisplayName = "sts.idp.ciws display name";
            role.ProtocolsSupported.Add(new Uri("http://docs.oasis-open.org/wsfed/federation/200706"));
            role.ValidUntil = DateTime.Now.AddDays(1);
            role.ClaimTypesOffered.Add(new DisplayClaim(Microsoft.IdentityModel.Claims.ClaimTypes.Role));
            role.ClaimTypesOffered.Add(new DisplayClaim(Microsoft.IdentityModel.Claims.ClaimTypes.Name));

            role.Keys.Add(
                new KeyDescriptor()
                {
                    KeyInfo =
                        new SecurityKeyIdentifier(
                            new X509RawDataKeyIdentifierClause(SimpleSecurityTokenServiceConfiguration.SigningCertificate)
                        ),
                    Use = KeyType.Signing
                }
            );


            ent.RoleDescriptors.Add(role);

            // Serialize
            var serializer = new MetadataSerializer();
            var xe = new XDocument();
            var writer = xe.CreateWriter();
            serializer.WriteMetadata(writer, ent);
            writer.Close();
            context.Response.Write(xe.ToString(SaveOptions.DisableFormatting));     
        }
     
        public bool IsReusable {
            get {
                return false;
            }
        }
    }
}